//+------------------------------------------------------------------+
//|                                          Perevertysh20240512.mq5 |
//|                                                          Voronin |
//|                                      Virtual Trailing Stop 2.mq5 |
//|                              Copyright © 2020, Vladimir Karputov |
//|                     https://www.mql5.com/ru/market/product/43516 |
//|                                           http://wmua.ru/slesar/ |
//|                                                           Gerbut |
//|                                           andriygerbut@gmail.com |
//+------------------------------------------------------------------+
#property version   "1.00"
#property version   "1.000"
#property strict

// Libraries:
//+------------------------------------------------------------------+
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
//#include <SecAuxLib.mqh>
#include <MT4Orders.mqh>

CPositionInfo     k_pos;
CSymbolInfo       m_symbol;
CTrade            m_trade;

//+------------------------------------------------------------------+
// Input parameters:
//+------------------------------------------------------------------+
enum ENUM_SYMBOL
{
    EURUSD  = 0,
    GPBUSD  = 1,
    USDCHF  = 2,
    BTCUSD  = 3,
    USDCAD  = 4,
    AUDCAD  = 5,
    NAZUSD  = 6,
    AUDCHF  = 7,
    AUDJPY  = 8,
    AUDUSD  = 9,
    BITCOIN = 10,
    XAUUSD  = 11,
    XAGUSD  = 12
};

input ENUM_SYMBOL     SYMBOL;
input ENUM_TIMEFRAMES TimeFrame        = PERIOD_CURRENT;
input double          LotSize          = 0.01; 
input ulong           Magic_m          = 9191;        // M1-9991,H1-9960,H4-9240,D1-1440,perevertysh-9191
input ulong           InpDeviation     = 10;          // Deviation, in points (1.00045-1.00055=10 points)
input ulong           volSpred         = 5;           // во сколько раз волатильность больше спреда
input double          TakeProfit       = 0.0;
input int             Kratnist         = 2; 
input bool            UseBreakeven     = true;        // Использовать ли трейлинг
//input double          StopLossLock     = spred;
//input double          StopLoss         = spred;
input double          InpTrailingStart = 0.02;        // Minimum profit to start trailing, in pips (1.00045-1.00055=1 pips)
input double          InpTrailingStep  = 1.0;         // Trailing Step, in pips (1.00045-1.00055=1 pips)
//+------------------------------------------------------------------+
double   StopLossLock                  = 0;
double   StopLoss                      = 0;
double   TakeProfitLock                = MathRound(StopLoss / Kratnist); // Take Profit, in pips (1.00045-1.00055=1 pips)
double   SymbolPoint                   = SymbolInfoDouble(Symbol(),SYMBOL_POINT); // значение пункта цены
double   ask                           = NormalizeDouble(SymbolInfoDouble(Symbol(), SYMBOL_ASK),_Digits);
double   bid                           = NormalizeDouble(SymbolInfoDouble(Symbol(), SYMBOL_BID),_Digits);
double   Tick_Size                     = SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_SIZE); // минимально возможное изменение цены
ulong    m_position_ticket             = 0;
ulong    ClosedTicket                  = 0;
MqlTradeRequest m_request;                         // структура торгового запроса
MqlTradeResult m_result;                           // структура результата торгового запроса
double   m_stop_loss                   = 0.0;      // Stop Loss                        -> double
double   m_StopLoss_Lock               = 0.0;      // Stop Loss Lock                   -> double
double   m_take_profit                 = 0.0;      // Take Profit                      -> double
double   m_take_profit_Lock            = 0.0;      // Take Profit Lock                 -> double
double   m_trailing_stop               = 0.0;      // Trailing Stop                    -> double
double   m_trailing_start              = 0.0;      // Minimum profit to start trailing -> double
double   m_trailing_step               = 0.0;      // Trailing Step                    -> double
double   m_adjusted_point;                         // point value adjusted for 3 or 5 points
int      resultat                      = 0;
ulong    MS                            = 1000;
ulong    TFMS                          = PeriodSeconds(/*Period()*/ PERIOD_M1) * MS;

//+------------------------------------------------------------------+
// Methods-Events:                                                   |
//+------------------------------------------------------------------+
int OnInit()
{
   m_trade.SetExpertMagicNumber(Magic_m);
   if(!m_symbol.Name(Symbol()))                    // sets symbol name
     {
      Print(__FILE__," ",__FUNCTION__,", ERROR: CSymbolInfo.Name");
      return(INIT_FAILED);
     }
   RefreshRates();
//---
   m_trade.SetMarginMode();
   m_trade.SetTypeFillingBySymbol(m_symbol.Name());
   m_trade.SetDeviationInPoints(InpDeviation);

   m_adjusted_point        = SymbolPoint;                                    //m_symbol.Point()*digits_adjust;
   m_stop_loss             = StopLoss                 * m_adjusted_point;
   m_StopLoss_Lock         = StopLossLock             * m_adjusted_point;
   m_take_profit           = TakeProfit               * m_adjusted_point;
   m_take_profit_Lock      = TakeProfitLock           * m_adjusted_point;
   m_trailing_start        = InpTrailingStart;         //* m_adjusted_point; прибыль
   m_trailing_step         = InpTrailingStep          * m_adjusted_point;
//---
        EventSetMillisecondTimer(100);
        return (INIT_SUCCEEDED);
}
    
void OnTick()
{
   OnTimer();
}

void OnTimer()
{
   /*int         total            = PositionsTotal();
   for(int i=total-1; i>=0; i--)
   {
      k_pos.SelectByIndex(i);
      mposition_ticket = PositionGetTicket(i);
      string m_position_symbol       = k_pos.Symbol();
      long   m_position_magic        = k_pos.Magic();
      Print("total = ",(string)total,", mposition_ticket = ",(string)mposition_ticket,", m_position_magic = ",(string)m_position_magic);*/
        MqlTradeRequest tradeRequest = { };
        MqlTick last_tick;
        MqlTick current_tick;
        SymbolInfoTick(Symbol(),last_tick);
        SymbolInfoTick(Symbol(),current_tick);
        double ask0             = current_tick.ask;                               //SymbolInfoDouble(Symbol(), SYMBOL_ASK);
        double bid0             = current_tick.bid;
        double spred            = ask0 - bid0;
               m_stop_loss      = spred*volSpred;
               m_StopLoss_Lock  = spred*volSpred;
        ulong  time0            = current_tick.time_msc;
        ulong  timeOpen         = PositionGetInteger(POSITION_TIME_MSC);
        ENUM_POSITION_TYPE Type = (ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE); // тип позиции
        double Price            = PositionGetDouble(POSITION_PRICE_OPEN);
        double Tp               = PositionGetDouble(POSITION_TP);
        m_position_ticket       = PositionGetInteger(POSITION_TICKET);
        double lt               = PositionGetDouble(POSITION_VOLUME); // объем позиции
        ulong  magic            = PositionGetInteger(POSITION_MAGIC); // MagicNumber позиции
        double Vol10SecUp1      = 0;
        double Vol10SecLow1     = 0;
        double Vol1MUp1         = 0;
        double Vol1MLow1        = 0;

MqlTick ticks[];
MqlTick tick;
SymbolInfoTick(Symbol(),tick);
  MqlTick mqlTick10Sec[], symbTick10Sec; // объявили переменные
  SymbolInfoTick(_Symbol, symbTick10Sec); // это последний тик
  int copied10Sec = CopyTicksRange(_Symbol, mqlTick10Sec, COPY_TICKS_INFO, symbTick10Sec.time_msc, symbTick10Sec.time_msc-10000); // цены за последние 10 секунд
  double bid_arr10Sec[];
  ArrayResize(bid_arr10Sec,copied10Sec,100000);
  double ask_arr10Sec[];
  ArrayResize(ask_arr10Sec,copied10Sec,100000);
  for(int i = 0; i < copied10Sec; i++)
  {
            bid_arr10Sec[i] = mqlTick10Sec[i].bid;
            ask_arr10Sec[i] = mqlTick10Sec[i].ask;
     double bid10Sec        = NormalizeDouble(bid_arr10Sec[0], _Digits); // первая цена bid за последние 10 секунд
     double ask10Sec        = NormalizeDouble(ask_arr10Sec[0], _Digits); // первая цена ask за последние 10 секунд
     int    IndMaxBid   = ArrayMaximum(bid_arr10Sec, 0, ArraySize(bid_arr10Sec)-1); // Индекс элемента массива bid_arr10Sec с максимальной ценой
     double highestBid  = bid_arr10Sec[IndMaxBid];                      // Самая высокая цена bid за последние 10 секунд
     int    IndMinBid   = ArrayMinimum(bid_arr10Sec, 0, ArraySize(bid_arr10Sec)-1); // Индекс элемента массива bid_arr10Sec с минимальной ценой
     double lowestBid   = bid_arr10Sec[IndMinBid];                      // Самая низкая цена bid за последние 10 секунд
     int    IndMaxAsk   = ArrayMaximum(ask_arr10Sec, 0, ArraySize(ask_arr10Sec)-1); // Индекс элемента массива ask_arr10Sec с максимальной ценой
     double highestAsk  = ask_arr10Sec[IndMaxAsk];                      // Самая высокая цена ask за последние 10 секунд
     int    IndMinAsk   = ArrayMinimum(ask_arr10Sec, 0, ArraySize(ask_arr10Sec)-1); // Индекс элемента массива ask_arr10Sec с минимальной ценой
     double lowestAsk   = ask_arr10Sec[IndMinAsk];                     // Самая низкая цена ask за последние 10 секунд
     double highestHigh = (highestBid+highestAsk)/2;               // Средняя наибольшего ask и наибольшего bid элемента
     double lowestLow   = (lowestBid+lowestAsk)/2;                 // Средняя наименьшего ask и наименьшего bid элемента
     if(highestHigh - (ask0 + bid0 + bid10Sec + ask10Sec)/4 >= (ask0 + bid0 + bid10Sec + ask10Sec)/4 - lowestLow)
     {
        Vol10SecUp1 = highestHigh - (ask0 + bid0 + bid10Sec + ask10Sec)/4;
        Vol10SecLow1 = 0;
     }
     else if(highestHigh - (ask0 + bid0 + bid10Sec + ask10Sec)/4 < (ask0 + bid0 + bid10Sec + ask10Sec)/4 - lowestLow)
     {
        Vol10SecLow1 = (ask0 + bid0 + bid10Sec + ask10Sec)/4 - lowestLow;
        Vol10SecUp1 = 0;
     }
  }
  
  MqlTick mqlTick1M[], symbTick1M; // объявили переменные
  SymbolInfoTick(_Symbol, symbTick1M); // это последний тик
  int copied1M = CopyTicksRange(_Symbol, mqlTick1M, COPY_TICKS_INFO, symbTick1M.time_msc, symbTick1M.time_msc-60000); // цены за последнюю минуту
  double bid_arr1M[];
  ArrayResize(bid_arr1M,copied1M,100000);
  double ask_arr1M[];
  ArrayResize(ask_arr1M,copied1M,100000);
  for(int i = 0; i < copied1M; i++)
  {
            bid_arr1M[i] = mqlTick1M[i].bid;
            ask_arr1M[i] = mqlTick1M[i].ask;
     double bid1M        = NormalizeDouble(bid_arr1M[0], _Digits); // первая цена bid за последнюю минуту
     double ask1M        = NormalizeDouble(ask_arr1M[0], _Digits); // первая цена ask за последнюю минуту
     int    IndMaxBid   = ArrayMaximum(bid_arr1M, 0, ArraySize(bid_arr1M)-1); // Индекс элемента массива bid_arr1M с максимальной ценой
     double highestBid  = bid_arr1M[IndMaxBid];                      // Самая высокая цена bid за последнюю минуту
     int    IndMinBid   = ArrayMinimum(bid_arr1M, 0, ArraySize(bid_arr1M)-1); // Индекс элемента массива bid_arr1M с минимальной ценой
     double lowestBid   = bid_arr1M[IndMinBid];                      // Самая низкая цена bid за последнюю минуту
     int    IndMaxAsk   = ArrayMaximum(ask_arr1M, 0, ArraySize(ask_arr1M)-1); // Индекс элемента массива ask_arr1M с максимальной ценой
     double highestAsk  = ask_arr1M[IndMaxAsk];                      // Самая высокая цена ask за последнюю минуту
     int    IndMinAsk   = ArrayMinimum(ask_arr1M, 0, ArraySize(ask_arr1M)-1); // Индекс элемента массива ask_arr1M с минимальной ценой
     double lowestAsk   = ask_arr1M[IndMinAsk];                     // Самая низкая цена ask за последнюю минуту
     double highestHigh = (highestBid+highestAsk)/2;               // Средняя наибольшего ask и наибольшего bid элемента
     double lowestLow   = (lowestBid+lowestAsk)/2;                 // Средняя наименьшего ask и наименьшего bid элемента
     if(highestHigh - (ask0 + bid0 + bid1M + ask1M)/4 >= (ask0 + bid0 + bid1M + ask1M)/4 - lowestLow)
     {
        Vol1MUp1 = highestHigh - (ask0 + bid0 + bid1M + ask1M)/4;
        Vol1MLow1 = 0;
     }
     else if(highestHigh - (ask0 + bid0 + bid1M + ask1M)/4 < (ask0 + bid0 + bid1M + ask1M)/4 - lowestLow)
     {
        Vol1MLow1 = (ask0 + bid0 + bid1M + ask1M)/4 - lowestLow;
        Vol1MUp1 = 0;
     }
  }

  MqlTick mqlTick[], symbTick; // объявили переменные
  CopyTicksRange(_Symbol, mqlTick, COPY_TICKS_INFO, symbTick.time_msc, symbTick.time_msc-100); // цены за последний период
  for(int i = 0; i < ArraySize(mqlTick); i++)
  {
     double bid1 = NormalizeDouble(mqlTick[0].bid, _Digits); // первая цена bid за последний период
     double ask1 = NormalizeDouble(mqlTick[0].ask, _Digits); // первая цена ask за последний период
      if(!PositionSelect(Symbol())) // Нет открытых позиций
        {
//Print("spred=",DoubleToString(spred,_Digits)," Vol1MLow1=",DoubleToString(Vol1MLow1,_Digits)," Vol1MUp1=",DoubleToString(Vol1MUp1,_Digits)," Vol10SecLow1=",DoubleToString(Vol10SecLow1,_Digits)," Vol10SecUp1=",DoubleToString(Vol10SecUp1,_Digits));
            if(spred < 0)
            {
                m_trade.Buy(LotSize,Symbol(),ask0,NULL,NULL,NULL);
                m_trade.Sell(LotSize,Symbol(),bid0,NULL,NULL,NULL);
Print("1 бай,селл,",DoubleToString(spred,_Digits)," < 0");
                //ModifyTrailingStop();
            }
            if(spred >= 0 && ((spred*volSpred <= Vol10SecUp1 && Vol10SecLow1 == 0) || (spred*volSpred <= Vol1MUp1 && Vol1MLow1 == 0)) && (ask0 > ask1 && bid0 > bid1))
            {
               m_trade.Buy(LotSize,Symbol(),ask0,ask0 - spred*volSpred,NULL,NULL);
Print("2 бай,",DoubleToString(spred,_Digits),">=0 && ((",DoubleToString((spred*volSpred),_Digits),"<=",DoubleToString(Vol10SecUp1,_Digits)," && ",DoubleToString(Vol10SecLow1,_Digits),"=0) || (",DoubleToString((spred*volSpred),_Digits),"<=",DoubleToString(Vol1MUp1,_Digits)," && ",DoubleToString(Vol1MLow1,_Digits),"=0)) && (",DoubleToString(ask0,_Digits),">",DoubleToString(ask1,_Digits)," && ",DoubleToString(bid0,_Digits),">",DoubleToString(bid1,_Digits),")");
               ModifyTrailingStop();
            }
            if(spred >= 0 && ((spred*volSpred <= Vol10SecLow1 && Vol10SecUp1 == 0) || (spred*volSpred <= Vol1MLow1 && Vol1MUp1 == 0)) && (ask0 < ask1 && bid0 < bid1))
            {
               m_trade.Sell(LotSize,Symbol(),bid0,bid0 + spred*volSpred,NULL,NULL);
Print("3 селл,",DoubleToString(spred,_Digits),">=0 && ((",DoubleToString((spred*volSpred),_Digits),"<=",DoubleToString(Vol10SecLow1,_Digits)," && ",DoubleToString(Vol10SecUp1,_Digits),"=0) || (",DoubleToString((spred*volSpred),_Digits),"<=",DoubleToString(Vol1MLow1,_Digits)," && ",DoubleToString(Vol1MUp1,_Digits),"=0)) && (",DoubleToString(ask0,_Digits),"<",DoubleToString(ask1,_Digits)," && ",DoubleToString(bid0,_Digits),"<",DoubleToString(bid1,_Digits),")");
               ModifyTrailingStop();
            }
        }
        if(PositionSelect(Symbol())) // Есть открытые позиции
        {
            //Print("Есть открытые позиции");
            if(spred < 0 && Type == POSITION_TYPE_BUY && bid0 > Price)
            {
               Print("1 spred < 0 && Type == POSITION_TYPE_BUY && bid0 > Price");
               if(m_trade.PositionClose(m_position_ticket)) Print("1 закрыли");
            }
            if(spred < 0 && Type == POSITION_TYPE_SELL && ask0 < Price)
            {
               Print("2 spred < 0 && Type == POSITION_TYPE_SELL && ask0 < Price");
               if(m_trade.PositionClose(m_position_ticket)) Print("2 закрыли");
            }
            //if(spred >= 0 && Type == POSITION_TYPE_BUY && (bid0 + spred < bid1 || ask0 + spred < ask1))
            //{
            //   Print("3 spred>0,BUY,",DoubleToString((bid0+spred),_Digits)," < ",DoubleToString(bid1,_Digits),"||",DoubleToString((ask0+spred),_Digits)," < ",DoubleToString(ask1,_Digits));
            //   if(m_trade.PositionClose(m_position_ticket)) Print("3 закрыли");
            //}
            //if(spred >= 0 && Type == POSITION_TYPE_SELL && (ask0 > ask1 + spred || bid0 > bid1 + spred))
            //{
            //   Print("4 spred>0,SELL,",DoubleToString(ask0,_Digits)," > ",DoubleToString((ask1 + spred),_Digits),"||",DoubleToString(bid0,_Digits)," > ",DoubleToString((bid1 + spred),_Digits));
            //   if(m_trade.PositionClose(m_position_ticket)) Print("4 закрыли");
            //}
        }
     }
}

//+------------------------------------------------------------------+
//| Refreshes the symbol quotes data                                 |
//+------------------------------------------------------------------+
bool RefreshRates(void)
  {
   if(!m_symbol.RefreshRates())
     {
      Print("RefreshRates error");
      return(false);
     }
//--- protection against the return value of "zero"
   if(m_symbol.Ask()==0 || m_symbol.Bid()==0)
      return(false);
//---
   return(true);
  }

//+------------------------------------------------------------------+
//  ModifyTrailingStop                                               |
//+------------------------------------------------------------------+
int ModifyTrailingStop()
{
   Print("Вошли в ModifyTrailingStop()");
       resultat               = 0;
   int total                  = PositionsTotal();                                         // количество открытых позиций   
   for(int i=total-1; i>=0; i--)                                                          // перебор всех открытых позиций
   {
      k_pos.SelectByIndex(i);
            m_position_ticket = PositionGetTicket(i);                                      // тикет позиции
      //Print("m_position_ticket= "+IntegerToString(m_position_ticket,0));
      string position_symbol  = PositionGetString(POSITION_SYMBOL);                        // символ 
      int m_digits=(int)SymbolInfoInteger(Symbol(),SYMBOL_DIGITS);                         // количество знаков после запятой
      ulong  magic            = PositionGetInteger(POSITION_MAGIC);                        // MagicNumber позиции
      double Tick_Value       = SymbolInfoDouble(Symbol(),SYMBOL_TRADE_TICK_VALUE);
      ENUM_POSITION_TYPE Type = (ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);     // тип позиции 
      double Price            = PositionGetDouble(POSITION_PRICE_OPEN);
      double lt               = PositionGetDouble(POSITION_VOLUME);                        // объем позиции
      double SLBuy            = 0;                                       // цена(!) стоплосса
      double SLSell           = 0;                                       // цена(!) стоплосса
      double Tp               = PositionGetDouble(POSITION_TP); 
      double sl0              = 0;
      double sl1              = 0;
      MqlTick last_tick;
      MqlTick current_tick;
      SymbolInfoTick(Symbol(),last_tick);
      SymbolInfoTick(Symbol(),current_tick);
      double ask1             = last_tick.ask;      
      double bid1             = last_tick.bid;
      double ask0             = current_tick.ask;
      double bid0             = current_tick.bid;
      double spred            = ask0 - bid0;
             m_stop_loss      = spred*volSpred;
             m_StopLoss_Lock  = spred*volSpred;
      double PrBuy            = PositionGetDouble(POSITION_PROFIT); //lt*Tick_Value*(bid1-ask0)/Tick_Size;
      double PrSell           = PositionGetDouble(POSITION_PROFIT); //lt*Tick_Value*(bid0-ask1)/Tick_Size;
      MqlTick ticks[];
      ulong  t1               = current_tick.time_msc;
      ulong  t0               = PositionGetInteger(POSITION_TIME_MSC);
//Print("t0 = ", (string)(t0), " t1 = ", (string)(t1), " _Period = ", (string)_Period);
int copied=CopyTicksRange(Symbol(),ticks,COPY_TICKS_INFO,t0,t1); // цены со времени открытия позиции
double ask_arr[];
ArrayResize(ask_arr,copied,100000);
double bid_arr[];
ArrayResize(bid_arr,copied,100000);

for(int i = 0; i < copied; i++)
{
   ask_arr[i] = ticks[i].ask;
   bid_arr[i] = ticks[i].bid;
}
int IndMaxAsk = ArrayMaximum(ask_arr, 0, copied-1); // Индекс элемента массива ask_arr с максимальной ценой
double highestAsk = ask_arr[IndMaxAsk]; // Самая высокая цена ask за период
int IndMinBid = ArrayMinimum(bid_arr, 0, copied-1); // Индекс элемента массива bid_arr с минимальной ценой
double lowestBid = bid_arr[IndMinBid]; // Самая низкая цена bid за период
      
      if(lt == LotSize && Type == POSITION_TYPE_BUY)SLBuy = Price - m_stop_loss; // цена стоплосса Бай
      if(lt == LotSize * Kratnist && Type == POSITION_TYPE_BUY)SLBuy = Price - m_StopLoss_Lock; // цена стоплосса замка Бай
      if(lt == LotSize && Type == POSITION_TYPE_SELL)SLSell = Price + m_stop_loss; // цена стоплосса Селл
      if(lt == LotSize * Kratnist && Type == POSITION_TYPE_SELL)SLSell = Price + m_StopLoss_Lock; // цена стоплосса замка Селл
      
      if(PositionSelect(Symbol()) && magic == Magic_m) // есть открытая позиция
      {
         //Print("PositionSelect(Symbol()) = ", (string)PositionSelect(Symbol()), " Magic_m = ", (string)Magic_m);
         switch(Type)
         {
            case POSITION_TYPE_BUY:
             if(UseBreakeven)
             {
                if(Price + (Price - SLBuy) + m_trailing_step > ask0 && ask0 > Price)
                {
                      Print("PrBuy1= " + DoubleToString(PrBuy,2));
                      if(PrBuy >= m_trailing_start)
                      {
                               sl0 = NormalizeDouble(Price + m_trailing_step, _Digits);
                               Print("sl0= " + DoubleToString(sl0,_Digits)
                               /*+" Tp= " + DoubleToString(Tp,_Digits)*/,
                               " ask0= ",DoubleToString(ask0,_Digits),
                               " Price= ",DoubleToString(Price,_Digits),
                               " SLBuy= ",DoubleToString(SLBuy,_Digits));
                               if(!m_trade.PositionModify(m_position_ticket, sl0, Tp))
                               {
                                  Print("ошибка модификации Бай по первому условию");
                               }
                               if(ask0 <= sl0 &&  lt == LotSize)
                               {
                                  m_trade.PositionClose(m_position_ticket);
                                  resultat = 0;
                                  //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                               }
                      }
                      resultat = 0;
                      //Print("Трейлингстоп resultat по бай1 = ", IntegerToString(resultat));
                }
                if(ask0 > Price + (Price - SLBuy) + m_trailing_step)
                {
                      Print("PrBuy2= " + DoubleToString(PrBuy,2));
                      if(PrBuy >= m_trailing_start)
                      {
                            sl0 = NormalizeDouble(ask0 - (Price - SLBuy), _Digits);
                            sl1 = NormalizeDouble(highestAsk - (Price - SLBuy), _Digits);
Print("sl0= " + DoubleToString(sl0,_Digits)," sl1= " + DoubleToString(sl1,_Digits));
                            if(sl0 < sl1)
                            {   
                               sl0 = sl1;
                            }
Print("sl0= " + DoubleToString(sl0,_Digits)," sl1= " + DoubleToString(sl1,_Digits)," SLBuy= " + DoubleToString(SLBuy,_Digits)," spred*volSpred= ",DoubleToString(spred*volSpred,_Digits));
                            if(!m_trade.PositionModify(m_position_ticket, sl0, Tp))
                            {
                               Print("ошибка модификации Бай по третьему условию");
                            }
                            if(ask0 <= sl0 && lt == LotSize)
                            {
                               m_trade.PositionClose(m_position_ticket);
                               resultat = 0;
                               //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                            }
                      }
                      resultat = 0;
                      //Print("Трейлингстоп resultat по бай2 = ", IntegerToString(resultat));
                }
                if(Price - (Price - SLBuy) < ask0 && ask0 <= Price)
                {
                   sl0 = NormalizeDouble(Price - (Price - SLBuy), _Digits);
                   if(!m_trade.PositionModify(m_position_ticket, sl0, Tp))//Print("модифицировать sl не надо, Бай 3-е условие");
                   resultat = 0;
Print("sl0= " + DoubleToString(sl0,_Digits)," SLBuy= " + DoubleToString(SLBuy,_Digits)," spred*volSpred= ",DoubleToString(spred*volSpred,_Digits));
                   if(ask0 <= sl0 && lt == LotSize)
                   {
                      m_trade.PositionClose(m_position_ticket);
                      resultat = 0;
                      //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                   }
                }
                if(ask0 < Price - (Price - SLBuy) && lt == LotSize)
                {
                   ClosedTicket = m_position_ticket;
                   if(!m_trade.PositionClose(m_position_ticket))
                   {
                      Print("ошибка принудительного закрытия Бай по стоплоссу");
                      m_trade.PositionClose(m_position_ticket);
                   }
                   resultat = 0;
Print("ask0= " + DoubleToString(ask0,_Digits)," Price - (Price - SLBuy)= " + DoubleToString(Price - (Price - SLBuy),_Digits));
                   //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                }
                else if(ask0 < Price - (Price - SLBuy) && lt == LotSize * Kratnist)
                {
                   m_trade.PositionClose(m_position_ticket);
                   resultat = 0;
                }
             }
          break;
          case POSITION_TYPE_SELL:
             if(UseBreakeven)
             {
                if(Price - (SLSell - Price) - m_trailing_step < bid0 && bid0 < Price)
                {
                      Print("PrSell1= " + DoubleToString(PrSell,2));
                      if(PrSell >= m_trailing_start)
                      {
                               sl0 = NormalizeDouble(Price - m_trailing_step, _Digits);
                               Print("sl0= " + DoubleToString(sl0,_Digits)
                               /*+" Tp= " + DoubleToString(Tp,_Digits)*/,
                               " bid0= ",DoubleToString(bid0,_Digits),
                               " Price= ",DoubleToString(Price,_Digits),
                               " SLSell= ",DoubleToString(SLSell,_Digits));
                               if(!m_trade.PositionModify(m_position_ticket, sl0, Tp))
                               {
                                  Print("ошибка модификации Селл по первому условию");
                               }
                               if(bid0 >= sl0 && lt == LotSize)
                               {
                                  m_trade.PositionClose(m_position_ticket);
                                  resultat = 0;
                                  //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                               }
                      }
                      resultat = 0;
                      //Print("Трейлингстоп resultat по селл1 = ", IntegerToString(resultat));
                }
                if(bid0 < Price - (SLSell - Price) - m_trailing_step)
                {
                      Print("PrSell2= " + DoubleToString(PrSell,2));
                      if(PrSell >= m_trailing_start)
                      {
                               sl0 = NormalizeDouble(bid0 + (SLSell - Price), _Digits);
                               sl1 = NormalizeDouble(lowestBid + (SLSell - Price), _Digits);
Print("sl0= " + DoubleToString(sl0,_Digits)," sl1= " + DoubleToString(sl1,_Digits));
                               if(sl0 > sl1)
                               {
                                  sl0 = sl1;
                               }
Print("sl0= " + DoubleToString(sl0,_Digits)," sl1= " + DoubleToString(sl1,_Digits)," SLSell= " + DoubleToString(SLSell,_Digits)," spred*volSpred= ",DoubleToString(spred*volSpred,_Digits));
                               if(!m_trade.PositionModify(m_position_ticket, sl0, Tp))
                               {
                                  Print("ошибка модификации Селл по второму условию");
                               }
                               if(bid0 >= sl0 && lt == LotSize)
                               {
                                  m_trade.PositionClose(m_position_ticket);
                                  resultat = 0;
                                  //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                               }
                      }
                      resultat = 0;
                      //Print("Трейлингстоп resultat по селл2 = ", IntegerToString(resultat));
                }
                if(Price >= bid0 > Price + (SLSell - Price))
                {
                       sl0 = NormalizeDouble(Price + (SLSell - Price), _Digits);
                       if(!m_trade.PositionModify(m_position_ticket, sl0, Tp))//Print("модифицировать sl не надо, Селл 3-е условие");
                       resultat = 0;
Print("sl0= " + DoubleToString(sl0,_Digits)," SLSell= " + DoubleToString(SLSell,_Digits)," spred*volSpred= ",DoubleToString(spred*volSpred,_Digits));
                       if(bid0 >= sl0 && lt == LotSize)
                       {
                          m_trade.PositionClose(m_position_ticket);
                          resultat = 0;
                          //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                       }
                }
                if(bid0 > Price + (SLSell - Price) && lt == LotSize)
                {
                      ClosedTicket = m_position_ticket;
                      if(!m_trade.PositionClose(m_position_ticket))
                      {
                         Print("ошибка принудительного закрытия Селл по стоплоссу");
                         m_trade.PositionClose(m_position_ticket);
                      }
                      resultat = 0;
Print("bid0= " + DoubleToString(bid0,_Digits)," Price + (SLSell - Price)= " + DoubleToString(Price + (SLSell - Price),_Digits));
                      //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
                }
                else if(bid0 > Price + (SLSell - Price) && lt == LotSize * Kratnist)
                {
                   m_trade.PositionClose(m_position_ticket);
                   resultat = 0;
                }
           }
           break;
      }
      //Sleep(10);
    }
  }
  //Print("Трейлингстоп resultat = ", IntegerToString(resultat));
  return(resultat);
  //if(!PositionSelect(Symbol()))ModifyAndLockOrders();
}

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   EventKillTimer();
}
